#! /usr/bin/env python3

from sys import argv
from pprint import pprint
import json
import yaml

import altium

def main(files):
    record = {}
    for file in files:
        print(f"read {file}")
        with open(file, "rb") as file:
            file = altium.OleFileIO(file)
            stream = file.openstream("FileHeader")
            objects = altium.iter_records(stream)
            for i, o in enumerate(objects):
                props = altium.parse_properties(stream, o)
                if not i:
                    i = "Header"
                else:
                    i = format(i - 1)
                print(f"{i}: {props}")
                id_ = props.get_int('OWNERINDEX')
                d = record.setdefault(id_, {'id': id_})
                match props.get_int('RECORD'):
                    case altium.Record.DESIGNATOR:
                        value = props['TEXT']
                        d['designator'] = value
                    case altium.Record.PARAMETER:
                        name = props['NAME']
                        value = props['TEXT']
                        d[name] = value
    components = {}
    def value_checker(_):
        if isinstance(_, str):
            try:
                if '.' in _:
                    return float(_)
                else:
                    return int(_)
            except ValueError:
                pass
        return _
    for d in record.values():
        if 'designator' in d:
            components[d['designator']] = {key.capitalize(): value_checker(value) for key, value in d.items()}
    with open('components.json', 'w', encoding='utf8') as fh:
        json.dump(
            components,
            fh,
            indent=4,
            sort_keys=True,
            ensure_ascii=False,
        )
    with open('components.yml', 'w') as fh:
        yaml.dump(
            components,
            fh,
            indent=4,
            sort_keys=True,
            encoding='utf8',
            allow_unicode=True
        )

if __name__ == "__main__":
    try:
        main(argv[1:])
    except (KeyboardInterrupt, ConnectionError):
        raise SystemExit(1)
